// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import createHash from 'create-hash';
import createHmac from 'create-hmac';
import { createHash as hashNode } from 'crypto';
import utils from './utils.js';

export default class Hash {
    static sha(message) {
        return Hash.shaBuffer(message).toString('base64');
    }

    static shaBuffer(message) {
        var data = !utils.isBuffer(message) ? Buffer.from(message) : message;
        return createHash('sha256').update(data).digest();
    }

    static sha512Buffer(message) {
        var data = !utils.isBuffer(message) ? Buffer.from(message) : message;
        return createHash('sha512').update(data).digest();
    }

    static hmac(data, secret) {
        return Hash.hmacBuffer(data, secret).toString('base64');
    }
    
    static hmacBuffer(data, secret) {
        secret = !utils.isBuffer(secret) ? Buffer.from(secret) : secret;
        return createHmac('sha256', secret).update(data).digest();
    }

    static ripeBuffer(message) {
        var data = !utils.isBuffer(message) ? Buffer.from(message) : message;
        return hashNode('ripemd160').update(data).digest();
    }
}