import assert from 'assert';
import elliptic, { ec as EC } from 'elliptic';
import Utils from '../src/utils';
import SecretShare from '../src/secretShare';
import C25519Point from '../src/c25519Point';
import { getBN } from '../src/bnInput';

/** @type {elliptic.curve.base} */
var curve = new EC('p384').curve;

describe('OPRF', function () {
    it('Should work with ECC', function () {
        for (let i = 0; i < 10; i++) {
            var n = C25519Point.n;
    
            var pass = Utils.random(1, n.minus(1)).toString(16);
            var k = Utils.random(1, n.minus(1));
            var r = Utils.random(1, n.minus(1));
    
            var g = C25519Point.fromString(pass);
            var gK1 = g.multiply(k);
            var gR = g.multiply(r);
    
            var gRK1 = gR.multiply(k);
            var gK2 = gRK1.multiply(r.modInv(n));
    
            assert.equal(gK1.eq(gK2), true)
        }
    });

    it('Should work with Threshold ECC', function () {
        for (let i = 0; i < 10; i++) {
            var n = C25519Point.n;
    
            var pass = Utils.random(1, n.minus(1)).toString(16);
            var k = Utils.random(1, n.minus(1));
            var r = Utils.random(1, n.minus(1));
    
            var t = 5;
            /** @type {bigInt.BigInteger[][]} */
            var [shrId, shrKey] = SecretShare.share(k, t, t, n);
    
            var g = C25519Point.fromString(pass);
            var gK1 = g.multiply(k);
    
            var gR = g.multiply(r);
            var gRKs = shrKey.map((ki, i) => SecretShare.getLi(shrId[i], shrId, n).times(ki).mod(n)).map(ki => gR.multiply(getBN(ki)));
    
            var gRK2 = gRKs.reduce((rki, sum) => rki.add(sum));
            var gK2 = gRK2.multiply(r.modInv(n));
    
            assert.equal(gK1.eq(gK2), true)
        }
    });

    var p = C25519Point.p;
    var SerializeTest = function SerializeTest(compact) {
        it('Should serialize and deserialize c25519 point when compact: ' + compact, function () {
            for (let i = 0; i < 20; i++) {
                var gr1 = C25519Point.g.times(Utils.random(1, p.minus(1)));
                var gr2 = C25519Point.from(gr1.toArray(compact), compact);
        
                assert.equal(gr1.eq(gr2), true)
            }
        });
    };

    SerializeTest(true);
    SerializeTest(false);
});
