import assert from 'assert';
import utils from '../src/utils.js';
import PaillierKey from '../src/paillier/paillierKey.js';
import PaillierCipher from '../src/paillier/paillierCipher.js';

describe('PaillierKey', function () {
    describe('encrypt()', function () {
        describe('decrypt()', function () {
            it('should operate homomorphically', async function () {
                for (let i = 0; i < 10; i++) {
                    var skey = await PaillierKey.random(128);
                    var pkey = skey.public();

                    var m1 = utils.random(1, pkey.n.subtract(1));
                    var m2 = utils.random(1, pkey.n.subtract(1));

                    var chipherM1 = await PaillierCipher.encrypt(m1, pkey);
                    var plainM1 = await PaillierCipher.decrypt(chipherM1, skey);

                    var cipher2m1 = chipherM1.add(chipherM1);
                    var plain2m1 = await PaillierCipher.decrypt(cipher2m1, skey);

                    var cipherM1AndM2 = chipherM1.add(m2);
                    var plainM1AndM2 = await PaillierCipher.decrypt(cipherM1AndM2, skey);

                    var cipherM2 = await PaillierCipher.encrypt(m2, pkey);
                    var cipherMdiff = chipherM1.minus(cipherM2);
                    var plainMdiff = await PaillierCipher.decrypt(cipherMdiff, skey);

                    var cipherM1SubM2 = chipherM1.minus(m2);
                    var plainM1SubM2 = await PaillierCipher.decrypt(cipherM1SubM2, skey);

                    var cipherM1M2 = chipherM1.times(m2);
                    var plainM1M2 = await PaillierCipher.decrypt(cipherM1M2, skey);

                    assert.equal(true, m1.equals(plainM1));
                    assert.equal(true, m1.add(m1).mod(pkey.n).equals(plain2m1));
                    assert.equal(true, m1.add(m2).mod(pkey.n).equals(plainM1AndM2));
                    assert.equal(true, utils.mod(m1.minus(m2), pkey.n).equals(plainMdiff));
                    assert.equal(true, utils.mod(m1.minus(m2), pkey.n).equals(plainM1SubM2));
                    assert.equal(true, m1.times(m2).mod(pkey.n).equals(plainM1M2));
                }
            });
        });
    });
});
